
Object subclass: #OGLTexture
	instanceVariableNames: 'form'
	classVariableNames: ''
	poolDictionaries: 'OpenGLConstants'
	category: 'OpenGL-Core'!

!OGLTexture commentStamp: 'bf 5/29/2012 17:39' prior: 0!
This is a wrapper for a Form, mimicking Croquet's TForms.!

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 17:04'!
bMipmap
	"answer true if we want mipmapping"
	^false! !

!OGLTexture methodsFor: 'updating' stamp: 'bf 5/29/2012 17:22'!
boundingBox
	^form boundingBox! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 17:44'!
form: aForm
	form := aForm asFormOfDepth: 32.
! !

!OGLTexture methodsFor: 'updating' stamp: 'bf 5/29/2012 17:42'!
invalidate: aRectangle
	"ignore - we always upload the whole texture"! !

!OGLTexture methodsFor: 'testing' stamp: 'bf 5/29/2012 16:31'!
isStatic
	^false! !

!OGLTexture methodsFor: 'testing' stamp: 'bf 5/29/2012 12:29'!
isThumb
	^false! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 17:27'!
magFilter
	^GLLinear! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 17:05'!
minFilter
	^ self bMipmap
		ifTrue: [GLLinearMipmapLinear]
		ifFalse: [GLLinear]! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 17:41'!
objectID
	"fake - this is used to detect changed forms. If objectID changes, form is uploaded again"
	^form! !

!OGLTexture methodsFor: 'updating' stamp: 'bf 5/29/2012 17:42'!
resetUpdateRect
	"ignore - we always upload the whole texture"! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 17:41'!
sha
	"fake - this is used as dictionary key to detect identical forms"
	^self! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 16:30'!
target
	^ GLTexture2d! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 16:31'!
textureForm
	^form! !

!OGLTexture methodsFor: 'accessing' stamp: 'bf 5/29/2012 16:48'!
trueExtent
	^ form width asLargerPowerOfTwo @ form height asLargerPowerOfTwo! !

!OGLTexture methodsFor: 'updating' stamp: 'bf 5/29/2012 17:42'!
updateRect
	"we always upload the whole texture"
	^form boundingBox! !

!OpenGL class methodsFor: 'example' stamp: 'bf 5/29/2012 17:46'!
exampleTex	"OpenGL exampleTex"
	"A very simple OpenGL example with texture"

	| ogl frames startTime deltaTime framesPerSec bounds font tex |
	font := StrikeFont familyName: 'Atlanta' pointSize: 11.
	bounds := 0@0 extent: 400@400.
	ogl := OpenGL newIn: bounds.
	ogl ifNil:[^self error: 'Unable to create renderer'].
	[frames := 0.
	startTime := Time millisecondClockValue.
	tex := OGLTexture new form: (Display copy: (0@0 extent: 140@80)).
	ogl installTexture: tex.
	[Sensor anyButtonPressed] whileFalse:[
		"start counting at second frame since first frame is penalized
		by the upload of the bitmap font and texture outside of ogl."
		frames = 1 ifTrue:[startTime := Time millisecondClockValue].
		ogl beginFrame.

	"--- this is the actual scene content ---"

		ogl glDisable: GLDepthTest.	"for the simple example only"
		ogl glDisable: GLLighting.		"for the simple example only"

		ogl glClearColor: 1.0 with: 1.0 with: 1.0 with: 1.0.
		ogl glClear: GLColorBufferBit.

		ogl glRotatef: 0.1 with: 0.0 with: 0.0 with: 1.0.
		ogl glColor3f: 1.0 with: 1.0 with: 1.0.
		ogl bindTexture: tex.
		ogl enableTexture: tex.

		ogl glBegin: GLPolygon.
			ogl glTexCoord2f: 0.0 with: 1.0.
			ogl glVertex2f: -0.7 with: -0.4.
			ogl glTexCoord2f: 1.0 with: 1.0.
			ogl glVertex2f:  0.7 with: -0.4.
			ogl glTexCoord2f: 1.0 with: 0.0.
			ogl glVertex2f:  0.7 with:  0.4.
			ogl glTexCoord2f: 0.0 with: 0.0.
			ogl glVertex2f: -0.7 with:  0.4.
		ogl glEnd.

		ogl disableTexture: tex.

	"--- here is the 2d overlay setup ---"

		ogl glMatrixMode: GLProjection.
		ogl glPushMatrix.
		ogl glLoadIdentity.
		ogl glMatrixMode: GLModelview.
		ogl glPushMatrix.
		ogl glLoadIdentity.
		ogl glTranslated: -1 with: 1 with: 0.0.
		ogl glScaled: (2.0 / bounds width) with: (-2.0 / bounds height) with: 1.0.
		ogl glDisable: GLDepthTest.
		ogl glEnable: GLBlend.
		ogl glBlendFunc: GLOne with: GLOneMinusSrcAlpha.

	"--- here is the 2d overlay rendering ---"
		deltaTime := Time millisecondsSince: startTime.
		framesPerSec := frames * 1000 / (deltaTime max: 1) asFloat.
		
		"@@@@: Fixme. It appears as if #drawString: depends on glColor being set.
		Makes no sense but I'm not going to figure this out - probably some mishap
		wrt. GLLighting being disabled."
		ogl glColor3f: 0.0 with: 0.0 with: 0.0.
		ogl drawString: frames printString, ' frames: ', (framesPerSec printShowingMaxDecimalPlaces: 1), ' fps'
			at: 0@font height@0 font: font color: Color black.

		ogl glDisable: GLBlend.
		ogl glMatrixMode: GLModelview.
		ogl glPopMatrix.
		ogl glMatrixMode: GLProjection.
		ogl glPopMatrix.
		ogl glMatrixMode: GLModelview.

	"--- end the end frame operations"

		ogl endFrame.
		ogl swapBuffers.
		frames := frames + 1.
	].
	] ensure:[ogl destroy]. ogl! !
